fx.layer

Composable dependency injection and resource lifecycle management inspired by Effect-ts and ZIO.

Layers model recipes for acquiring and releasing environmental context and services. Layers can be composed horizontally with merge>, vertically with compose>, and provided to effect pipelines with provide-layer> or managed with start-layer! and launch-sync!.

Composable dependency injection and resource lifecycle management inspired by Effect-ts and ZIO.

Layers model recipes for acquiring and releasing environmental context and services.
Layers can be composed horizontally with `merge>`, vertically with `compose>`,
and provided to effect pipelines with `provide-layer>` or managed with `start-layer!`
and `launch-sync!`.
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries